{
  "id": 1,
  "developer": [
    {"name":"deva"}
  ],
  "platforms": [
    {"name":"PC"}
  ],
  "rating": {
    "code": "E",
    "name": "Everyone"
  },
  "releaseDate": "2015-01-01",
  "title": "A Game"
}
{
  "id": 2,
  "developer": [
    {"name":"devb"}
  ],
  "platforms": [
    {"name":"Xbox"},
    {"name":"Playstation"}
  ],
  "rating": {
    "code": "E",
    "name": "Everyone"
  },
  "releaseDate": "2016-01-01",
  "title": "B Game"
}
